from configparser import ConfigParser
import re


class MyConfigParser(ConfigParser):

    def getlist(self, section, key, seperator=','):
        return re.split(' *%s *' % seperator, self[section][key])

    def getmodule(self, section, key):
        path, clsname = self[section][key].rsplit(".", 1)
        module = __import__(path, globals(), locals(), [clsname])
        return getattr(module, clsname)


def create_config(stream):
    config = MyConfigParser(
        comment_prefixes=[';', '//'],
        dict_type=dict
    )
    config.read_file(stream)

    return config


'''
Created on 01-04-2013

@author: firemark
'''
import unittest
import config
config.nick = "Test"
from ..modules import get_args_from_text, re_cmd, check_type
from ..utils.types import range_int


class ModuleTest(unittest.TestCase):

    @staticmethod
    def func_test(bot, arg1, arg2,
                  karg:range_int(0, 10)=0,
                  akarg:int=2):
        pass
    func_test.varkw = None
    func_test.varargs = None

    def test_cmd(self):
        for msg in (
                    "Test: cmd testing spaces",
                    ".cmd testing spaces"
                    ):
            cmd, text = re_cmd.match(msg).groups()
            self.assertEqual(cmd, "cmd", msg)
            self.assertEqual(text, "testing spaces", msg)

        for msg in (
                    "Test: another_cmd",
                    ".another_cmd"
                    ):
            cmd, text = re_cmd.match(msg).groups()
            self.assertEqual(cmd, "another_cmd", msg)
            self.assertEqual(text, None, msg)

    def test_get_args(self):
        max_args = 2

        for text, cor_args, cor_kwargs in (
                        ("firemark tiramo",
                         ["firemark", "tiramo"],
                         {}),

                        ("tiramo firemark with spaces",
                         ["tiramo", "firemark with spaces"],
                         {}),

                        ("tiramo firemark with spaces karg: 2 akarg: 5",
                         ["tiramo", "firemark with spaces"],
                         {"karg": "2", "akarg": "5"}),

                        ('"tiramo with spaces" firemark karg:" spaces "',
                         ["tiramo with spaces", "firemark"],
                         {"karg": " spaces "}),

                        ('karg: firemark arg2= 20 10 ',
                         ["10"],
                         {"karg": "firemark", "arg2": "20"}),

                            ):
            args, kwargs = get_args_from_text(text, max_args)
            self.assertListEqual(args, cor_args, text)
            self.assertDictEqual(kwargs, cor_kwargs, text)

    def test_check_type(self):
        for args, kwargs, result in (
                        #without kwargs
                         (["t1", "t2"], {},
                          {"arg1": "t1", "arg2": "t2", "karg": 0, "akarg": 2}),
                        #with optionals
                         (["t", "t", "-1", "-1"], {},
                          {"arg1": "t", "arg2": "t", "karg": 0, "akarg": -1}),
                        #mixing
                         (["ta", "t"], {"karg": "5", "akarg": "-1"},
                          {"arg1": "ta", "arg2": "t", "karg": 5, "akarg": -1}),
                        #another
                         (["t", "t"], {"karg": "15", "akarg": "10"},
                          {"arg1": "t", "arg2": "t", "karg": 10, "akarg": 10}),
                        #without args
                         ([], {"arg1": "a", "arg2": "c", "karg": "5", "akarg": "10"},
                          {"arg1": "a", "arg2": "c", "karg": 5, "akarg": 10}),
                         ):
            self.assertEqual(
                             check_type(args, kwargs, ModuleTest.func_test),
                             result
                             )

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


#!/usr/bin/python3

from ..utils import register
from time import sleep
from ..utils.types import range_int, is_chan

from ..config import create_config
from sys import argv
import asyncio

@register(cmd='reload', admin_required=True)
def reload(bot, module):
    bot.protocol.importer.reload(module)
    bot.reply('Done!')


@register(cmd='load-plugin', admin_required=True)
def load_plugin(bot, module, name):
    bot.protocol.importer.load(name, module)
    bot.reply('Done!')


@register(cmd='remove-plugin', admin_required=True)
def remove_plugin(bot, name):
    bot.protocol.importer.remove(name)
    bot.reply('Done!')


@register(cmd='reload-config', admin_required=True)
def reload_config(bot, reload_plugins='yes'):
    with open(argv[1]) as f:
        bot.protocol.config = create_config(f)

    if reload_plugins != 'yes':
        bot.reply('Done!')
        return

    importer = bot.protocol.importer
    new_plugins = bot.protocol.config.items('plugins')
    loaded_plugins = set(importer.plugins.keys())
    plugins_to_remove = loaded_plugins - set(n for n, m in new_plugins)

    for name in plugins_to_remove:
        importer.remove(name)

    for name, module in bot.protocol.config.items('plugins'):
        if name not in loaded_plugins:
            bot.protocol.importer.load(name, module)

    bot.reply('Done!')


@register(cmd='kick', admin_required=True)
def kick(bot, who, chan=None, why=''):
    if bot.private and not chan:
        return
    bot.kick(who, why, chan)


@register(cmd='join', admin_required=True)
def join(bot, chan):
    bot.command('JOIN', chan)


@register(cmd='eutanazja', admin_required=True)
def quit_bot(bot):
    bot.command('QUIT', 'why? :(')


@register(cmd='say', admin_required=True)
def say(bot, msg, nick=None, chan:is_chan=None):
    if bot.private and not(chan or nick):
        return

    if chan and nick:
        bot.say(nick + ": " + msg, chan)
    else:
        bot.say(msg, chan or nick)

@register(cmd='op', admin_required=True)
def op(bot, nick=None, chan:is_chan=None):
    if bot.private and not chan:
        return

    bot.mode("+o", nick or bot.user.nick, chan)


@register(cmd='part', admin_required=True)
def part(bot, chan:is_chan, why=None):
    if bot.private and not chan:
        return
    bot.command_msg('PART', chan, why)


@register(cmd='rocket', admin_required=True)
def rocket(bot, nick, n:range_int(0, 10)=3, chan:is_chan=None):
    if bot.private and not chan:
        return

    enemy = nick
    if n == 0 or enemy == bot.config['__nick__']:
        enemy = bot.user.nick

    @asyncio.coroutine
    def timer(): 
        for i in range(n):
            bot.say(str(n - i) + '...')
            yield from asyncio.sleep(1)
        bot.say('FIRE!')
        whois = yield from bot.protocol.whois(enemy)
        yield from asyncio.sleep(1)
        prefix = "*!%s@%s" % (whois.realname or '*', whois.host or '*')
        bot.time_ban(n * 2, why='Kaboom!', who=enemy, chan=chan, prefix=prefix)
        
    asyncio.async(timer())



class NoSuchNickError(Exception):

    nick = None
    
    def __init__(self, nick):
        self.nick = nick

    def __repr__(self):
        return 'NoSuchNickError(%s)' % self.nick

    def __str__(self):
        return repr(self)


import asyncio
import sys
import traceback
from random import random

def loop(time_config_key, default):
    def outter(func):
        @asyncio.coroutine
        def inner(protocol, plugin, future):
            while True:
                if future.cancelled():
                    break
                cfg = protocol.config['plugin:%s' % plugin.name]
                time_to_sleep = cfg.getint(time_config_key, default)
                if time_to_sleep == 0:
                    break

                if protocol.ready:
                    #special random delay
                    yield from asyncio.sleep(random() * 3)
                    try:
                        func(protocol, plugin, cfg)
                    except:
                        traceback.print_exc(file=sys.stdout)


                yield from asyncio.sleep(time_to_sleep)
        inner.is_loop = True
        return inner
    return outter





from ..utils import register
from .. import format
from ..format import color
from datetime import timedelta, datetime, timezone

from aiohttp import request
import asyncio
import re

URL_API = 'http://api.openweathermap.org/data/2.5/forecast'
URL_API_LONG = URL_API + '/daily'

re_date = re.compile(r'([+-]?\d+)([dh])')

ICON_TO_UTF = {
    '01d': color('☀', color.yellow),
    '02d': color('☀', color.yellow),
    '03d': color('☁', color.white),
    '04d': color('☁', color.white),
    '09d': color('☔', color.light_blue),
    '10d': color('☔', color.light_blue),
    '11d': color('⚡', color.yellow),
    '13d': color('❄', color.white),
    '01n': color('☾', color.white),
    '02n': color('☾', color.white),
    '03n': color('☁', color.white),
    '04n': color('☁', color.white),
    '09n': color('☔', color.light_blue),
    '10n': color('☔', color.light_blue),
    '11n': color('⚡', color.yellow),
    '13n': color('❄', color.white),
}

@register(cmd='weather')
def weather(bot, city, day=None):
    dt = check_and_return_datetime(day)
    timestamp = dt.replace(tzinfo=timezone.utc).timestamp()

    delta_days = (dt - datetime.now()).days
    is_long_forecast = (delta_days > 5)

    def get_weather():
        if not is_long_forecast:
            response = yield from request('GET', URL_API, params={
                'q': city,
                'units': 'metric',
                'lang': 'pl'
            })
        else:
            response = yield from request('GET', URL_API_LONG, params={
                'q': city,
                'units': 'metric',
                'lang': 'pl',
                'cnt': min(delta_days, 16)
            })
        data_list = (yield from response.json())["list"]
        key = lambda x: abs(x['dt'] - timestamp)
        data = sorted(data_list, key=key)[0]
        try:
            weather = data['weather'][0]
        except IndexError:
            weather = {'description': '', 'icon': ''}

        if not is_long_forecast:
            temperature = data['main']['temp']
        else:
            if dt.hour < 6:
                period = 'night'
            elif dt.hour < 10:
                period = 'morn'
            elif dt.hour < 18:
                period = 'day'
            elif dt.hour < 22:
                period = 'eve'
            else:
                period = 'night'
            temperature = data['temp'][period]
        bot.reply('{city}: {temp} °C {icon} {desc}'.format(
            city=format.bold(city),
            temp=temperature,
            desc=weather['description'],
            icon=ICON_TO_UTF.get(weather['icon'], '')
        ))

    asyncio.async(get_weather())


def check_and_return_datetime(day):
    dt = datetime.now()
    if day is None:
        return dt

    dt_hour = datetime.now().replace(hour=12, minute=0, second=0)
    if day == "yesterday":
        return dt_hour - timedelta(days=1)
    if day == "tomorrow":
        return dt_hour + timedelta(days=1)

    match = re_date.match(day)
    if match is None:
        return dt
    number = int(match.group(1))
    if match.group(2) == 'd':
        return dt_hour + timedelta(days=number)
    else:
        return dt + timedelta(hours=number)



#from . import irc


class AbstractAuth(object):

    def add_permission(self, username, channel, level):
        raise NotImplementedError('add_permission')

    def check_permission(self, username, channel, level):
        raise NotImplementedError('check_permission')

    def auth(self):
        raise NotImplementedError('auth')


class NonAuth(AbstractAuth):

    def auth(self, protocol):
        pass

class QuakenetAuth(AbstractAuth):

    user = None
    passwd = None

    def __init__(self, user, passwd):
        self.user = user
        self.passwd = passwd

    def auth(self, protocol):
        protocol.say('Q@CServe.quakenet.org AUTH {0.user} {0.passwd}'.format(self))


class FreenodeAuth(AbstractAuth):

    passwd = None

    def __init__(self, passwd):
        self.passwd = passwd

    def auth(self, protocol):
        protocol.say('nickserv', 'identify %s' % self.passwd)


import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

requires = [
    'aiohttp==0.16.5'
]

setup(
    name='grazyna',
    version='0.5.3',
    download_url='https://github.com/firemark/grazyna/tarball/0.5.3',
    description='Grazyna The irc bot',
    long_description='Grazyna The irc bot',
    classifiers=[],
    author='Firemark',
    author_email='marpiechula@gmail.com',
    url='https://github.com/firemark/grazyna',
    keywords='irc socket bot'.split(),
    packages=['grazyna'] + ['grazyna.' + p for p in find_packages('grazyna')],
    package_data={'': ['default_config.ini']},
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    tests_require=requires,
    test_suite="grazyna.firemark.tests",
)



import random
from ..utils import register, create_help

#create_help('czy', '?czy <pytanie>')


@register(cmd='{cmd}')
def quotes(bot, ask=None):
    replies = get_replies(bot)
    reply = random.choice(replies)
    bot.reply(reply)


def get_replies(bot):
    """funny random quotes"""
    replies = bot.plugin.temp.get('replies')
    if replies is None:
        pathname = bot.config['file']
        with open(pathname) as f:
            replies = f.read().splitlines()
        bot.plugin.temp['replies'] = replies
    return replies
